import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_auth_helper/src/auth_provider.dart';

import '../auth_user.dart';
import '../credentials.dart';

/// see https://firebase.flutter.dev/docs/auth/usage/
class FirebaseAuthProvider implements AuthProvider {
  final firebaseAuth = FirebaseAuth.instance;

  AuthUser _toAuthUser(User? user) {
    if (user == null) return AuthUser.notLoggedIn();
    return AuthUser(user.uid,
        displayName: user.displayName ?? "",
        email: user.email ?? "",
        isLoggedIn: true);
  }

  @override
  Stream<AuthUser> get authUserChanged =>
      firebaseAuth.authStateChanges().map(_toAuthUser);

  @override
  AuthUser get currentAuthUser => _toAuthUser(firebaseAuth.currentUser);

  @override
  Future<String?> emailPasswordSignIn(LoginCredentials credentials) async {
    //see https://firebase.flutter.dev/docs/auth/usage/#emailpassword-registration--sign-in
    try {
      UserCredential userCredential = await FirebaseAuth.instance
          .signInWithEmailAndPassword(
              email: credentials.email, password: credentials.password);
      return null;
    } on FirebaseAuthException catch (e) {
      if (e.code == 'weak-password') {
        return 'The password provided is too weak.';
      } else if (e.code == 'email-already-in-use') {
        return 'The account already exists for that email.';
      } else {
        return ("Failed to login");
      }
    } catch (e) {
      return "Failed to login $e";
    }
  }

  @override
  Future<void> signOut() async {
    await FirebaseAuth.instance.signOut();
  }
}
